@@ -16,7 +16,7 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: avrdude.h 1294 2014-03-12 23:03:18Z joerg_wunsch $ */
+/* $Id: avrdude.h 1321 2014-06-13 20:07:40Z awachtler $ */
 
 #ifndef avrdude_h
 #define avrdude_h
@@ -28,6 +28,15 @@ extern int ovsigck;		/* override signature check (-F) */
 extern int verbose;		/* verbosity level (-v, -vv, ...) */
 extern int quell_progress;	/* quiteness level (-q, -qq) */
 
+int avrdude_message(const int msglvl, const char *format, ...);
+
+#define MSG_INFO    (0) /* no -v option, can be supressed with -qq */
+#define MSG_NOTICE  (1) /* displayed with -v */
+#define MSG_NOTICE2 (2) /* displayed with -vv, used rarely */
+#define MSG_DEBUG   (3) /* displayed with -vvv */
+#define MSG_TRACE   (4) /* displayed with -vvvv, show trace commuication */
+#define MSG_TRACE2  (5) /* displayed with -vvvvv */
+
 #if defined(WIN32NATIVE)
 
 #include "ac_cfg.h"
