@@ -17,7 +17,7 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: main.c 1294 2014-03-12 23:03:18Z joerg_wunsch $ */
+/* $Id: main.c 1347 2014-11-23 21:49:56Z joerg_wunsch $ */
 
 /*
  * Code to program an Atmel AVR device through one of the supported
@@ -33,6 +33,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdarg.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <limits.h>
@@ -44,17 +45,10 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 
-#include "avr.h"
-#include "config.h"
-#include "confwin.h"
-#include "fileio.h"
-#include "lists.h"
-#include "par.h"
-#include "pindefs.h"
+#include "avrdude.h"
+#include "libavrdude.h"
+
 #include "term.h"
-#include "safemode.h"
-#include "update.h"
-#include "pgm_type.h"
 
 
 /* Get VERSION from ac_cfg.h */
@@ -65,6 +59,19 @@ char   progbuf[PATH_MAX]; /* temporary buffer of spaces the same
                              length as progname; used for lining up
                              multiline messages */
 
+int avrdude_message(const int msglvl, const char *format, ...)
+{
+    int rc = 0;
+    va_list ap;
+    if (verbose >= msglvl) {
+        va_start(ap, format);
+        rc = vfprintf(stderr, format, ap);
+        va_end(ap);
+    }
+    return rc;
+}
+
+
 struct list_walk_cookie
 {
     FILE *f;
@@ -87,12 +94,14 @@ int    quell_progress; /* un-verebose output */
 int    ovsigck;     /* 1=override sig check, 0=don't */
 
 
+
+
 /*
  * usage message
  */
 static void usage(void)
 {
-  fprintf(stderr,
+  avrdude_message(MSG_INFO, 
  "Usage: %s [options]\n"
  "Options:\n"
  "  -p <partno>                Required. Specify AVR device.\n"
@@ -146,18 +155,18 @@ static void update_progress_tty (int percent, double etime, char *hdr)
   }
 
   if (hdr) {
-    fprintf (stderr, "\n");
+    avrdude_message(MSG_INFO, "\n");
     last = 0;
     header = hdr;
   }
 
   if (last == 0) {
-    fprintf(stderr, "\r%s | %s | %d%% %0.2fs", 
+    avrdude_message(MSG_INFO, "\r%s | %s | %d%% %0.2fs",
             header, hashes, percent, etime);
   }
 
   if (percent == 100) {
-    if (!last) fprintf (stderr, "\n\n");
+    if (!last) avrdude_message(MSG_INFO, "\n\n");
     last = 1;
   }
 
@@ -173,19 +182,19 @@ static void update_progress_no_tty (int percent, double etime, char *hdr)
   setvbuf(stderr, (char*)NULL, _IONBF, 0);
 
   if (hdr) {
-    fprintf (stderr, "\n%s | ", hdr);
+    avrdude_message(MSG_INFO, "\n%s | ", hdr);
     last = 0;
     done = 0;
   }
   else {
     while ((cnt > last) && (done == 0)) {
-      fprintf (stderr, "#");
+      avrdude_message(MSG_INFO, "#");
       cnt -=  2;
     }
   }
 
   if ((percent == 100) && (done == 0)) {
-    fprintf (stderr, " | 100%% %0.2fs\n\n", etime);
+    avrdude_message(MSG_INFO, " | 100%% %0.2fs\n\n", etime);
     last = 0;
     done = 1;
   }
@@ -374,19 +383,19 @@ int main(int argc, char * argv [])
 
   updates = lcreat(NULL, 0);
   if (updates == NULL) {
-    fprintf(stderr, "%s: cannot initialize updater list\n", progname);
+    avrdude_message(MSG_INFO, "%s: cannot initialize updater list\n", progname);
     exit(1);
   }
 
   extended_params = lcreat(NULL, 0);
   if (extended_params == NULL) {
-    fprintf(stderr, "%s: cannot initialize extended parameter list\n", progname);
+    avrdude_message(MSG_INFO, "%s: cannot initialize extended parameter list\n", progname);
     exit(1);
   }
 
   additional_config_files = lcreat(NULL, 0);
   if (additional_config_files == NULL) {
-    fprintf(stderr, "%s: cannot initialize additional config files list\n", progname);
+    avrdude_message(MSG_INFO, "%s: cannot initialize additional config files list\n", progname);
     exit(1);
   }
 
@@ -459,7 +468,7 @@ int main(int argc, char * argv [])
       case 'b': /* override default programmer baud rate */
         baudrate = strtol(optarg, &e, 0);
         if ((e == optarg) || (*e != 0)) {
-          fprintf(stderr, "%s: invalid baud rate specified '%s'\n",
+          avrdude_message(MSG_INFO, "%s: invalid baud rate specified '%s'\n",
                   progname, optarg);
           exit(1);
         }
@@ -467,8 +476,49 @@ int main(int argc, char * argv [])
 
       case 'B':	/* specify JTAG ICE bit clock period */
 	bitclock = strtod(optarg, &e);
-	if ((e == optarg) || (*e != 0) || bitclock == 0.0) {
-	  fprintf(stderr, "%s: invalid bit clock period specified '%s'\n",
+	if (*e != 0) {
+	  /* trailing unit of measure present */
+	  int suffixlen = strlen(e);
+	  switch (suffixlen) {
+	  case 2:
+	    if ((e[0] != 'h' && e[0] != 'H') || e[1] != 'z')
+	      bitclock = 0.0;
+	    else
+	      /* convert from Hz to microseconds */
+	      bitclock = 1E6 / bitclock;
+	    break;
+
+	  case 3:
+	    if ((e[1] != 'h' && e[1] != 'H') || e[2] != 'z')
+	      bitclock = 0.0;
+	    else {
+	      switch (e[0]) {
+	      case 'M':
+	      case 'm':		/* no Millihertz here :) */
+		bitclock = 1.0 / bitclock;
+		break;
+
+	      case 'k':
+		bitclock = 1E3 / bitclock;
+		break;
+
+	      default:
+		bitclock = 0.0;
+		break;
+	      }
+	    }
+	    break;
+
+	  default:
+	    bitclock = 0.0;
+	    break;
+	  }
+	  if (bitclock == 0.0)
+	    avrdude_message(MSG_INFO, "%s: invalid bit clock unit of measure '%s'\n",
+			    progname, e);
+	}
+	if ((e == optarg) || bitclock == 0.0) {
+	  avrdude_message(MSG_INFO, "%s: invalid bit clock period specified '%s'\n",
                   progname, optarg);
           exit(1);
         }
@@ -477,7 +527,7 @@ int main(int argc, char * argv [])
       case 'i':	/* specify isp clock delay */
 	ispdelay = strtol(optarg, &e,10);
 	if ((e == optarg) || (*e != 0) || ispdelay == 0) {
-	  fprintf(stderr, "%s: invalid isp clock delay specified '%s'\n",
+	  avrdude_message(MSG_INFO, "%s: invalid isp clock delay specified '%s'\n",
                   progname, optarg);
           exit(1);
         }
@@ -553,7 +603,7 @@ int main(int argc, char * argv [])
       case 'U':
         upd = parse_op(optarg);
         if (upd == NULL) {
-          fprintf(stderr, "%s: error parsing update operation '%s'\n",
+          avrdude_message(MSG_INFO, "%s: error parsing update operation '%s'\n",
                   progname, optarg);
           exit(1);
         }
@@ -579,12 +629,12 @@ int main(int argc, char * argv [])
         break;
 
       case 'y':
-        fprintf(stderr, "%s: erase cycle counter no longer supported\n",
+        avrdude_message(MSG_INFO, "%s: erase cycle counter no longer supported\n",
                 progname);
         break;
 
       case 'Y':
-        fprintf(stderr, "%s: erase cycle counter no longer supported\n",
+        avrdude_message(MSG_INFO, "%s: erase cycle counter no longer supported\n",
                 progname);
         break;
 
@@ -594,7 +644,7 @@ int main(int argc, char * argv [])
         break;
 
       default:
-        fprintf(stderr, "%s: invalid option -%c\n\n", progname, ch);
+        avrdude_message(MSG_INFO, "%s: invalid option -%c\n\n", progname, ch);
         usage();
         exit(1);
         break;
@@ -625,50 +675,38 @@ int main(int argc, char * argv [])
     }
   }
 
-  if (verbose) {
-    /*
-     * Print out an identifying string so folks can tell what version
-     * they are running
-     */
-    fprintf(stderr,
-            "\n%s: Version %s, compiled on %s at %s\n"
-            "%sCopyright (c) 2000-2005 Brian Dean, http://www.bdmicro.com/\n"
-	    "%sCopyright (c) 2007-2014 Joerg Wunsch\n\n",
-            progname, version, __DATE__, __TIME__, progbuf, progbuf);
-  }
-
-  if (verbose) {
-    fprintf(stderr, "%sSystem wide configuration file is \"%s\"\n",
+  /*
+   * Print out an identifying string so folks can tell what version
+   * they are running
+   */
+  avrdude_message(MSG_NOTICE, "\n%s: Version %s, compiled on %s at %s\n"
+                    "%sCopyright (c) 2000-2005 Brian Dean, http://www.bdmicro.com/\n"
+                    "%sCopyright (c) 2007-2014 Joerg Wunsch\n\n",
+                    progname, version, __DATE__, __TIME__, progbuf, progbuf);
+  avrdude_message(MSG_NOTICE, "%sSystem wide configuration file is \"%s\"\n",
             progbuf, sys_config);
-  }
 
   rc = read_config(sys_config);
   if (rc) {
-    fprintf(stderr,
-            "%s: error reading system wide configuration file \"%s\"\n",
-            progname, sys_config);
+    avrdude_message(MSG_INFO, "%s: error reading system wide configuration file \"%s\"\n",
+                    progname, sys_config);
     exit(1);
   }
 
   if (usr_config[0] != 0) {
-    if (verbose) {
-      fprintf(stderr, "%sUser configuration file is \"%s\"\n",
+    avrdude_message(MSG_NOTICE, "%sUser configuration file is \"%s\"\n",
               progbuf, usr_config);
-    }
 
     rc = stat(usr_config, &sb);
     if ((rc < 0) || ((sb.st_mode & S_IFREG) == 0)) {
-      if (verbose) {
-        fprintf(stderr,
-                "%sUser configuration file does not exist or is not a "
-                "regular file, skipping\n",
-                progbuf);
-      }
+      avrdude_message(MSG_NOTICE, "%sUser configuration file does not exist or is not a "
+                      "regular file, skipping\n",
+                      progbuf);
     }
     else {
       rc = read_config(usr_config);
       if (rc) {
-        fprintf(stderr, "%s: error reading user configuration file \"%s\"\n",
+        avrdude_message(MSG_INFO, "%s: error reading user configuration file \"%s\"\n",
                 progname, usr_config);
         exit(1);
       }
@@ -681,16 +719,13 @@ int main(int argc, char * argv [])
 
     for (ln1=lfirst(additional_config_files); ln1; ln1=lnext(ln1)) {
       p = ldata(ln1);
-      if (verbose) {
-        fprintf(stderr, "%sAdditional configuration file is \"%s\"\n",
-                progbuf, p);
-      }
+      avrdude_message(MSG_NOTICE, "%sAdditional configuration file is \"%s\"\n",
+                      progbuf, p);
 
       rc = read_config(p);
       if (rc) {
-        fprintf(stderr,
-                "%s: error reading additional configuration file \"%s\"\n",
-                progname, p);
+        avrdude_message(MSG_INFO, "%s: error reading additional configuration file \"%s\"\n",
+                        progname, p);
         exit(1);
       }
     }
@@ -701,67 +736,61 @@ int main(int argc, char * argv [])
     bitclock = default_bitclock;
   }
 
-  if (verbose) {
-    fprintf(stderr, "\n");
-  }
+  avrdude_message(MSG_NOTICE, "\n");
 
   if (partdesc) {
     if (strcmp(partdesc, "?") == 0) {
-      fprintf(stderr, "\n");
-      fprintf(stderr,"Valid parts are:\n");
+      avrdude_message(MSG_INFO, "\n");
+      avrdude_message(MSG_INFO, "Valid parts are:\n");
       list_parts(stderr, "  ", part_list);
-      fprintf(stderr, "\n");
+      avrdude_message(MSG_INFO, "\n");
       exit(1);
     }
   }
 
   if (programmer) {
     if (strcmp(programmer, "?") == 0) {
-      fprintf(stderr, "\n");
-      fprintf(stderr,"Valid programmers are:\n");
+      avrdude_message(MSG_INFO, "\n");
+      avrdude_message(MSG_INFO, "Valid programmers are:\n");
       list_programmers(stderr, "  ", programmers);
-      fprintf(stderr,"\n");
+      avrdude_message(MSG_INFO, "\n");
       exit(1);
     }
     if (strcmp(programmer, "?type") == 0) {
-      fprintf(stderr, "\n");
-      fprintf(stderr,"Valid programmer types are:\n");
+      avrdude_message(MSG_INFO, "\n");
+      avrdude_message(MSG_INFO, "Valid programmer types are:\n");
       list_programmer_types(stderr, "  ");
-      fprintf(stderr,"\n");
+      avrdude_message(MSG_INFO, "\n");
       exit(1);
     }
   }
 
 
   if (programmer[0] == 0) {
-    fprintf(stderr,
-            "\n%s: no programmer has been specified on the command line "
-            "or the config file\n",
-            progname);
-    fprintf(stderr,
-            "%sSpecify a programmer using the -c option and try again\n\n",
-            progbuf);
+    avrdude_message(MSG_INFO, "\n%s: no programmer has been specified on the command line "
+                    "or the config file\n",
+                    progname);
+    avrdude_message(MSG_INFO, "%sSpecify a programmer using the -c option and try again\n\n",
+                    progbuf);
     exit(1);
   }
 
   pgm = locate_programmer(programmers, programmer);
   if (pgm == NULL) {
-    fprintf(stderr,"\n");
-    fprintf(stderr,
-            "%s: Can't find programmer id \"%s\"\n",
-            progname, programmer);
-    fprintf(stderr,"\nValid programmers are:\n");
+    avrdude_message(MSG_INFO, "\n");
+    avrdude_message(MSG_INFO, "%s: Can't find programmer id \"%s\"\n",
+                    progname, programmer);
+    avrdude_message(MSG_INFO, "\nValid programmers are:\n");
     list_programmers(stderr, "  ", programmers);
-    fprintf(stderr,"\n");
+    avrdude_message(MSG_INFO, "\n");
     exit(1);
   }
 
   if (pgm->initpgm) {
     pgm->initpgm(pgm);
   } else {
-    fprintf(stderr,
-            "\n%s: Can't initialize the programmer.\n\n",
-            progname);
+    avrdude_message(MSG_INFO, "\n%s: Can't initialize the programmer.\n\n",
+                    progname);
     exit(1);
   }
 
@@ -774,15 +803,13 @@ int main(int argc, char * argv [])
 
   if (lsize(extended_params) > 0) {
     if (pgm->parseextparams == NULL) {
-      fprintf(stderr,
-              "%s: WARNING: Programmer doesn't support extended parameters,"
-              " -x option(s) ignored\n",
-              progname);
+      avrdude_message(MSG_INFO, "%s: WARNING: Programmer doesn't support extended parameters,"
+                      " -x option(s) ignored\n",
+                      progname);
     } else {
       if (pgm->parseextparams(pgm, extended_params) < 0) {
-        fprintf(stderr,
-              "%s: Error parsing extended parameter list\n",
-              progname);
+        avrdude_message(MSG_INFO, "%s: Error parsing extended parameter list\n",
+                        progname);
         exit(1);
       }
     }
@@ -806,33 +833,30 @@ int main(int argc, char * argv [])
   }
 
   if (partdesc == NULL) {
-    fprintf(stderr,
-            "%s: No AVR part has been specified, use \"-p Part\"\n\n",
-            progname);
-    fprintf(stderr,"Valid parts are:\n");
+    avrdude_message(MSG_INFO, "%s: No AVR part has been specified, use \"-p Part\"\n\n",
+                    progname);
+    avrdude_message(MSG_INFO, "Valid parts are:\n");
     list_parts(stderr, "  ", part_list);
-    fprintf(stderr, "\n");
+    avrdude_message(MSG_INFO, "\n");
     exit(1);
   }
 
 
   p = locate_part(part_list, partdesc);
   if (p == NULL) {
-    fprintf(stderr,
-            "%s: AVR Part \"%s\" not found.\n\n",
-            progname, partdesc);
-    fprintf(stderr,"Valid parts are:\n");
+    avrdude_message(MSG_INFO, "%s: AVR Part \"%s\" not found.\n\n",
+                    progname, partdesc);
+    avrdude_message(MSG_INFO, "Valid parts are:\n");
     list_parts(stderr, "  ", part_list);
-    fprintf(stderr, "\n");
+    avrdude_message(MSG_INFO, "\n");
     exit(1);
   }
 
 
   if (exitspecs != NULL) {
     if (pgm->parseexitspecs == NULL) {
-      fprintf(stderr,
-              "%s: WARNING: -E option not supported by this programmer type\n",
-              progname);
+      avrdude_message(MSG_INFO, "%s: WARNING: -E option not supported by this programmer type\n",
+                      progname);
       exitspecs = NULL;
     }
     else if (pgm->parseexitspecs(pgm, exitspecs) < 0) {
@@ -866,7 +890,7 @@ int main(int argc, char * argv [])
 
   if (avr_initmem(p) != 0)
   {
-    fprintf(stderr, "\n%s: failed to initialize memories\n",
+    avrdude_message(MSG_INFO, "\n%s: failed to initialize memories\n",
             progname);
     exit(1);
   }
@@ -881,15 +905,12 @@ int main(int argc, char * argv [])
     upd = ldata(ln);
     if (upd->memtype == NULL) {
       const char *mtype = (p->flags & AVRPART_HAS_PDI)? "application": "flash";
-      if (verbose >= 2) {
-        fprintf(stderr,
-                "%s: defaulting memtype in -U %c:%s option to \"%s\"\n",
-                progname,
-                (upd->op == DEVICE_READ)? 'r': (upd->op == DEVICE_WRITE)? 'w': 'v',
-                upd->filename, mtype);
-      }
+      avrdude_message(MSG_NOTICE2, "%s: defaulting memtype in -U %c:%s option to \"%s\"\n",
+                      progname,
+                      (upd->op == DEVICE_READ)? 'r': (upd->op == DEVICE_WRITE)? 'w': 'v',
+                      upd->filename, mtype);
       if ((upd->memtype = strdup(mtype)) == NULL) {
-        fprintf(stderr, "%s: out of memory\n", progname);
+        avrdude_message(MSG_INFO, "%s: out of memory\n", progname);
         exit(1);
       }
     }
@@ -899,42 +920,36 @@ int main(int argc, char * argv [])
    * open the programmer
    */
   if (port[0] == 0) {
-    fprintf(stderr, "\n%s: no port has been specified on the command line "
+    avrdude_message(MSG_INFO, "\n%s: no port has been specified on the command line "
             "or the config file\n",
             progname);
-    fprintf(stderr, "%sSpecify a port using the -P option and try again\n\n",
+    avrdude_message(MSG_INFO, "%sSpecify a port using the -P option and try again\n\n",
             progbuf);
     exit(1);
   }
 
   if (verbose) {
-    fprintf(stderr, "%sUsing Port                    : %s\n", progbuf, port);
-    fprintf(stderr, "%sUsing Programmer              : %s\n", progbuf, programmer);
+    avrdude_message(MSG_NOTICE, "%sUsing Port                    : %s\n", progbuf, port);
+    avrdude_message(MSG_NOTICE, "%sUsing Programmer              : %s\n", progbuf, programmer);
     if ((strcmp(pgm->type, "avr910") == 0)) {
-	  fprintf(stderr, "%savr910_devcode (avrdude.conf) : ", progbuf);
-      if(p->avr910_devcode)fprintf(stderr, "0x%x\n", p->avr910_devcode);
-	  else fprintf(stderr, "none\n");
-    }  
+	  avrdude_message(MSG_NOTICE, "%savr910_devcode (avrdude.conf) : ", progbuf);
+      if(p->avr910_devcode)avrdude_message(MSG_INFO, "0x%x\n", p->avr910_devcode);
+	  else avrdude_message(MSG_NOTICE, "none\n");
+    }
   }
 
   if (baudrate != 0) {
-    if (verbose) {
-      fprintf(stderr, "%sOverriding Baud Rate          : %d\n", progbuf, baudrate);
-    }
+    avrdude_message(MSG_NOTICE, "%sOverriding Baud Rate          : %d\n", progbuf, baudrate);
     pgm->baudrate = baudrate;
   }
 
   if (bitclock != 0.0) {
-    if (verbose) {
-      fprintf(stderr, "%sSetting bit clk period        : %.1f\n", progbuf, bitclock);
-    }
+    avrdude_message(MSG_NOTICE, "%sSetting bit clk period        : %.1f\n", progbuf, bitclock);
     pgm->bitclock = bitclock * 1e-6;
   }
 
   if (ispdelay != 0) {
-    if (verbose) {
-      fprintf(stderr, "%sSetting isp clock delay        : %3i\n", progbuf, ispdelay);
-    }
+    avrdude_message(MSG_NOTICE, "%sSetting isp clock delay        : %3i\n", progbuf, ispdelay);
     pgm->ispdelay = ispdelay;
   }
 
@@ -952,30 +967,28 @@ int main(int argc, char * argv [])
      * as outlined in appnote AVR053
      */
     if (pgm->perform_osccal == 0) {
-      fprintf(stderr,
-              "%s: programmer does not support RC oscillator calibration\n",
-	      progname);
+      avrdude_message(MSG_INFO, "%s: programmer does not support RC oscillator calibration\n",
+                      progname);
       exitrc = 1;
     } else {
-      fprintf(stderr, "%s: performing RC oscillator calibration\n", progname);
+      avrdude_message(MSG_INFO, "%s: performing RC oscillator calibration\n", progname);
       exitrc = pgm->perform_osccal(pgm);
     }
     if (exitrc == 0 && quell_progress < 2) {
-      fprintf(stderr,
-              "%s: calibration value is now stored in EEPROM at address 0\n",
-              progname);
+      avrdude_message(MSG_INFO, "%s: calibration value is now stored in EEPROM at address 0\n",
+                      progname);
     }
     goto main_exit;
   }
 
   if (verbose) {
     avr_display(stderr, p, progbuf, verbose);
-    fprintf(stderr, "\n");
+    avrdude_message(MSG_NOTICE, "\n");
     programmer_display(pgm, progbuf);
   }
 
   if (quell_progress < 2) {
-    fprintf(stderr, "\n");
+    avrdude_message(MSG_INFO, "\n");
   }
 
   exitrc = 0;
@@ -998,9 +1011,9 @@ int main(int argc, char * argv [])
    */
   init_ok = (rc = pgm->initialize(pgm, p)) >= 0;
   if (!init_ok) {
-    fprintf(stderr, "%s: initialization failed, rc=%d\n", progname, rc);
+    avrdude_message(MSG_INFO, "%s: initialization failed, rc=%d\n", progname, rc);
     if (!ovsigck) {
-      fprintf(stderr, "%sDouble check connections and try again, "
+      avrdude_message(MSG_INFO, "%sDouble check connections and try again, "
               "or use -F to override\n"
               "%sthis check.\n\n",
               progbuf, progbuf);
@@ -1013,9 +1026,8 @@ int main(int argc, char * argv [])
   pgm->rdy_led(pgm, ON);
 
   if (quell_progress < 2) {
-    fprintf(stderr,
-            "%s: AVR device initialized and ready to accept instructions\n",
-            progname);
+    avrdude_message(MSG_INFO, "%s: AVR device initialized and ready to accept instructions\n",
+                    progname);
   }
 
   /*
@@ -1033,7 +1045,7 @@ int main(int argc, char * argv [])
     if (init_ok) {
       rc = avr_signature(pgm, p);
       if (rc != 0) {
-        fprintf(stderr, "%s: error reading signature data, rc=%d\n",
+        avrdude_message(MSG_INFO, "%s: error reading signature data, rc=%d\n",
           progname, rc);
         exitrc = 1;
         goto main_exit;
@@ -1042,42 +1054,48 @@ int main(int argc, char * argv [])
   
     sig = avr_locate_mem(p, "signature");
     if (sig == NULL) {
-      fprintf(stderr,
-              "%s: WARNING: signature data not defined for device \"%s\"\n",
-              progname, p->desc);
+      avrdude_message(MSG_INFO, "%s: WARNING: signature data not defined for device \"%s\"\n",
+                      progname, p->desc);
     }
 
     if (sig != NULL) {
       int ff, zz;
 
       if (quell_progress < 2) {
-        fprintf(stderr, "%s: Device signature = 0x", progname);
+        avrdude_message(MSG_INFO, "%s: Device signature = 0x", progname);
       }
       ff = zz = 1;
       for (i=0; i<sig->size; i++) {
         if (quell_progress < 2) {
-          fprintf(stderr, "%02x", sig->buf[i]);
+          avrdude_message(MSG_INFO, "%02x", sig->buf[i]);
         }
         if (sig->buf[i] != 0xff)
           ff = 0;
         if (sig->buf[i] != 0x00)
           zz = 0;
       }
+      if (quell_progress < 2) {
+        AVRPART * part;
+
+        part = locate_part_by_signature(part_list, sig->buf, sig->size);
+        if (part) {
+          avrdude_message(MSG_INFO, " (probably %s)", part->id);
+        }
+      }
       if (ff || zz) {
         if (++attempt < 3) {
           waittime *= 5;
           if (quell_progress < 2) {
-              fprintf(stderr, " (retrying)\n");
+              avrdude_message(MSG_INFO, " (retrying)\n");
           }
           goto sig_again;
         }
         if (quell_progress < 2) {
-            fprintf(stderr, "\n");
+            avrdude_message(MSG_INFO, "\n");
         }
-        fprintf(stderr,
-                "%s: Yikes!  Invalid device signature.\n", progname);
+        avrdude_message(MSG_INFO, "%s: Yikes!  Invalid device signature.\n", progname);
         if (!ovsigck) {
-          fprintf(stderr, "%sDouble check connections and try again, "
+          avrdude_message(MSG_INFO, "%sDouble check connections and try again, "
                   "or use -F to override\n"
                   "%sthis check.\n\n",
                   progbuf, progbuf);
@@ -1086,7 +1104,7 @@ int main(int argc, char * argv [])
         }
       } else {
         if (quell_progress < 2) {
-          fprintf(stderr, "\n");
+          avrdude_message(MSG_INFO, "\n");
         }
       }
 
@@ -1094,12 +1112,11 @@ int main(int argc, char * argv [])
           sig->buf[0] != p->signature[0] ||
           sig->buf[1] != p->signature[1] ||
           sig->buf[2] != p->signature[2]) {
-        fprintf(stderr,
-                "%s: Expected signature for %s is %02X %02X %02X\n",
-                progname, p->desc,
-                p->signature[0], p->signature[1], p->signature[2]);
+        avrdude_message(MSG_INFO, "%s: Expected signature for %s is %02X %02X %02X\n",
+                        progname, p->desc,
+                        p->signature[0], p->signature[1], p->signature[2]);
         if (!ovsigck) {
-          fprintf(stderr, "%sDouble check chip, "
+          avrdude_message(MSG_INFO, "%sDouble check chip, "
                   "or use -F to override this check.\n",
                   progbuf);
           exitrc = 1;
@@ -1114,24 +1131,20 @@ int main(int argc, char * argv [])
        and extended fuse bytes as needed */
 
     rc = safemode_readfuses(&safemode_lfuse, &safemode_hfuse,
-                           &safemode_efuse, &safemode_fuse, pgm, p, verbose);
+                           &safemode_efuse, &safemode_fuse, pgm, p);
 
     if (rc != 0) {
 
 	  //Check if the programmer just doesn't support reading
   	  if (rc == -5)
 			{
-			  if (verbose > 0)
-				{
-				fprintf(stderr, "%s: safemode: Fuse reading not support by programmer.\n"
-                	            "              Safemode disabled.\n", progname);
-				}
-	  		safemode = 0;
+				avrdude_message(MSG_NOTICE, "%s: safemode: Fuse reading not support by programmer.\n"
+						"              Safemode disabled.\n", progname);
 			}
       else
 			{
 
-      		fprintf(stderr, "%s: safemode: To protect your AVR the programming "
+      		avrdude_message(MSG_INFO, "%s: safemode: To protect your AVR the programming "
             				    "will be aborted\n",
                					 progname);
       		exitrc = 1;
@@ -1147,11 +1160,10 @@ int main(int argc, char * argv [])
     if ((p->flags & AVRPART_HAS_PDI) && pgm->page_erase != NULL &&
         lsize(updates) > 0) {
       if (quell_progress < 2) {
-        fprintf(stderr,
-                "%s: NOTE: Programmer supports page erase for Xmega devices.\n"
-                "%sEach page will be erased before programming it, but no chip erase is performed.\n"
-                "%sTo disable page erases, specify the -D option; for a chip-erase, use the -e option.\n",
-                progname, progbuf, progbuf);
+        avrdude_message(MSG_INFO, "%s: NOTE: Programmer supports page erase for Xmega devices.\n"
+                        "%sEach page will be erased before programming it, but no chip erase is performed.\n"
+                        "%sTo disable page erases, specify the -D option; for a chip-erase, use the -e option.\n",
+                        progname, progbuf, progbuf);
       }
     } else {
       AVRMEM * m;
@@ -1166,11 +1178,10 @@ int main(int argc, char * argv [])
         if ((strcasecmp(m->desc, memname) == 0) && (upd->op == DEVICE_WRITE)) {
           erase = 1;
           if (quell_progress < 2) {
-            fprintf(stderr,
-                    "%s: NOTE: \"%s\" memory has been specified, an erase cycle "
-                    "will be performed\n"
-                    "%sTo disable this feature, specify the -D option.\n",
-                    progname, memname, progbuf);
+            avrdude_message(MSG_INFO, "%s: NOTE: \"%s\" memory has been specified, an erase cycle "
+                            "will be performed\n"
+                            "%sTo disable this feature, specify the -D option.\n",
+                            progname, memname, progbuf);
           }
           break;
         }
@@ -1184,12 +1195,11 @@ int main(int argc, char * argv [])
      * before the chip can accept new programming
      */
     if (uflags & UF_NOWRITE) {
-      fprintf(stderr,
-	      "%s: conflicting -e and -n options specified, NOT erasing chip\n",
-	      progname);
+      avrdude_message(MSG_INFO, "%s: conflicting -e and -n options specified, NOT erasing chip\n",
+                      progname);
     } else {
       if (quell_progress < 2) {
-      	fprintf(stderr, "%s: erasing chip\n", progname);
+      	avrdude_message(MSG_INFO, "%s: erasing chip\n", progname);
       }
       exitrc = avr_chip_erase(pgm, p);
       if(exitrc) goto main_exit;
@@ -1234,7 +1244,7 @@ int main(int argc, char * argv [])
     char yes[1] = {'y'};
 
     if (quell_progress < 2) {
-      fprintf(stderr, "\n");
+      avrdude_message(MSG_INFO, "\n");
     }
 
     //Restore the default fuse values
@@ -1242,23 +1252,22 @@ int main(int argc, char * argv [])
 
     /* Try reading back fuses, make sure they are reliable to read back */
     if (safemode_readfuses(&safemodeafter_lfuse, &safemodeafter_hfuse,
-                           &safemodeafter_efuse, &safemodeafter_fuse, pgm, p, verbose) != 0) {
+                           &safemodeafter_efuse, &safemodeafter_fuse, pgm, p) != 0) {
       /* Uh-oh.. try once more to read back fuses */
       if (safemode_readfuses(&safemodeafter_lfuse, &safemodeafter_hfuse,
-                             &safemodeafter_efuse, &safemodeafter_fuse, pgm, p, verbose) != 0) { 
-        fprintf(stderr,
-                "%s: safemode: Sorry, reading back fuses was unreliable. "
-                "I have given up and exited programming mode\n",
-                progname);
+                             &safemodeafter_efuse, &safemodeafter_fuse, pgm, p) != 0) {
+        avrdude_message(MSG_INFO, "%s: safemode: Sorry, reading back fuses was unreliable. "
+                        "I have given up and exited programming mode\n",
+                        progname);
         exitrc = 1;
-        goto main_exit;		  
+        goto main_exit;        
       }
     }
     
     /* Now check what fuses are against what they should be */
     if (safemodeafter_fuse != safemode_fuse) {
       fuses_updated = 1;
-      fprintf(stderr, "%s: safemode: fuse changed! Was %x, and is now %x\n",
+      avrdude_message(MSG_INFO, "%s: safemode: fuse changed! Was %x, and is now %x\n",
               progname, safemode_fuse, safemodeafter_fuse);
 
               
@@ -1273,11 +1282,11 @@ int main(int argc, char * argv [])
               
             /* Enough chit-chat, time to program some fuses and check them */
             if (safemode_writefuse (safemode_fuse, "fuse", pgm, p,
-                                    10, verbose) == 0) {
-                fprintf(stderr, "%s: safemode: and is now rescued\n", progname);
+                                    10) == 0) {
+                avrdude_message(MSG_INFO, "%s: safemode: and is now rescued\n", progname);
             }
             else {
-                fprintf(stderr, "%s: and COULD NOT be changed\n", progname);
+                avrdude_message(MSG_INFO, "%s: and COULD NOT be changed\n", progname);
                 failures++;
             }
       }
@@ -1286,7 +1295,7 @@ int main(int argc, char * argv [])
     /* Now check what fuses are against what they should be */
     if (safemodeafter_lfuse != safemode_lfuse) {
       fuses_updated = 1;
-      fprintf(stderr, "%s: safemode: lfuse changed! Was %x, and is now %x\n",
+      avrdude_message(MSG_INFO, "%s: safemode: lfuse changed! Was %x, and is now %x\n",
               progname, safemode_lfuse, safemodeafter_lfuse);
 
               
@@ -1301,11 +1310,11 @@ int main(int argc, char * argv [])
               
             /* Enough chit-chat, time to program some fuses and check them */
             if (safemode_writefuse (safemode_lfuse, "lfuse", pgm, p,
-                                    10, verbose) == 0) {
-                fprintf(stderr, "%s: safemode: and is now rescued\n", progname);
+                                    10) == 0) {
+                avrdude_message(MSG_INFO, "%s: safemode: and is now rescued\n", progname);
             }
             else {
-                fprintf(stderr, "%s: and COULD NOT be changed\n", progname);
+                avrdude_message(MSG_INFO, "%s: and COULD NOT be changed\n", progname);
                 failures++;
             }
       }
@@ -1314,7 +1323,7 @@ int main(int argc, char * argv [])
     /* Now check what fuses are against what they should be */
     if (safemodeafter_hfuse != safemode_hfuse) {
       fuses_updated = 1;
-      fprintf(stderr, "%s: safemode: hfuse changed! Was %x, and is now %x\n",
+      avrdude_message(MSG_INFO, "%s: safemode: hfuse changed! Was %x, and is now %x\n",
               progname, safemode_hfuse, safemodeafter_hfuse);
               
       /* Ask user - should we change them */
@@ -1326,11 +1335,11 @@ int main(int argc, char * argv [])
 
             /* Enough chit-chat, time to program some fuses and check them */
             if (safemode_writefuse(safemode_hfuse, "hfuse", pgm, p,
-                                    10, verbose) == 0) {
-                fprintf(stderr, "%s: safemode: and is now rescued\n", progname);
+                                    10) == 0) {
+                avrdude_message(MSG_INFO, "%s: safemode: and is now rescued\n", progname);
             }
             else {
-                fprintf(stderr, "%s: and COULD NOT be changed\n", progname);
+                avrdude_message(MSG_INFO, "%s: and COULD NOT be changed\n", progname);
                 failures++;
             }
       }
@@ -1339,7 +1348,7 @@ int main(int argc, char * argv [])
     /* Now check what fuses are against what they should be */
     if (safemodeafter_efuse != safemode_efuse) {
       fuses_updated = 1;
-      fprintf(stderr, "%s: safemode: efuse changed! Was %x, and is now %x\n",
+      avrdude_message(MSG_INFO, "%s: safemode: efuse changed! Was %x, and is now %x\n",
               progname, safemode_efuse, safemodeafter_efuse);
 
       /* Ask user - should we change them */
@@ -1351,24 +1360,24 @@ int main(int argc, char * argv [])
               
             /* Enough chit-chat, time to program some fuses and check them */
             if (safemode_writefuse (safemode_efuse, "efuse", pgm, p,
-                                    10, verbose) == 0) {
-                fprintf(stderr, "%s: safemode: and is now rescued\n", progname);
+                                    10) == 0) {
+                avrdude_message(MSG_INFO, "%s: safemode: and is now rescued\n", progname);
             }
             else {
-                fprintf(stderr, "%s: and COULD NOT be changed\n", progname);
+                avrdude_message(MSG_INFO, "%s: and COULD NOT be changed\n", progname);
                 failures++;
             }
        }
     }
 
     if (quell_progress < 2) {
-      fprintf(stderr, "%s: safemode: ", progname);
+      avrdude_message(MSG_INFO, "%s: safemode: ", progname);
       if (failures == 0) {
-        fprintf(stderr, "Fuses OK (E:%02X, H:%02X, L:%02X)\n",
+        avrdude_message(MSG_INFO, "Fuses OK (E:%02X, H:%02X, L:%02X)\n",
                 safemode_efuse, safemode_hfuse, safemode_lfuse);
       }
       else {
-        fprintf(stderr, "Fuses not recovered, sorry\n");
+        avrdude_message(MSG_INFO, "Fuses not recovered, sorry\n");
       }
     }
 
@@ -1396,7 +1405,7 @@ main_exit:
   }
 
   if (quell_progress < 2) {
-    fprintf(stderr, "\n%s done.  Thank you.\n\n", progname);
+    avrdude_message(MSG_INFO, "\n%s done.  Thank you.\n\n", progname);
   }
 
   return exitrc;
